/* 
 * Copyright (C) 2007 Pierre-Luc Beaudoin
 * 
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation; either 
 * version 2 of the License, or (at your option) any later 
 * version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef CLIP_H
#define CLIP_H

#include <string>

/**
 * @class Clip
 * @brief The represent a multimedia clip.  
 *
 * It can be an mp3, a video file or a picture. 
 */


class Clip {
  // Associations
  // Attributes
  private:
    std::string _title;
    std::string _artist;
    std::string _album;
    int _duration;
    std::string _uri;
    std::string _imagePath;
    bool _canSeek;
  // Operations
  public:
    Clip();
    /**
     * @brief Returns the title of the clip.
     * @return The title of the clip.
     */
    std::string getTitle (  );
    
    /**
     * @brief Sets the title of the clip.
     * @param t The title of the clip.
     */
    void setTitle ( std::string t );
    
    /**
     * @brief Returns the artist name of the clip.
     * @return The artist name of the clip.
     */
    std::string getArtist (  );
    
    /**
     * @brief Sets the artist name of the clip.
     * @param a The artist name of the clip.
     */
    void setArtist ( std::string a );
    
    /**
     * @brief Returns the albums title of the clip.
     * @return The albums title of the clip.
     */
    std::string getAlbum (  );
    
    /**
     * @brief Sets the albums title of the clip.
     * @param a The albums title of the clip.
     */
    void setAlbum ( std::string a );
    
    /**
     * @brief Returns the duration of the clip.
     * @return The duration of the clip in seconds.
     */
    int getDuration (  );
    
    /**
     * @brief Sets the duration of the clip.
     * @param d The duration of the clip in seconds.
     */
    void setDuration ( int d );
    
    /**
     * @brief Sets the URI of the clip.
     * @param uri The URI of the clip in seconds.
     */
    std::string getURI (  );
    
    /**
     * @brief Sets the duration of the clip.
     * @param d The duration of the clip in seconds.
     */
    void setURI ( std::string uri );
    
    /**
     * @brief Returns the image of the clip.
     * @return The local path of the image of the clip.
     */
    std::string getImagePath (  );
    
    /**
     * @brief Sets the image of the clip.
     * @param path The local path of the image of the clip.
     */
    void setImagePath ( std::string path );
    
    bool canSeek ();
    bool setCanSeek (bool canSeek);
    
};

#endif
