/* 
 * Copyright (C) 2007 Pierre-Luc Beaudoin
 * 
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation; either 
 * version 2 of the License, or (at your option) any later 
 * version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */



#include "control.h"
#include "debug.h"
#include "sofa-i18n.h"
#include "settings.h"
#include "menus/selecters/mainmenu.h"
#include "modules/modulemanager.h"
#include "views/clutterview.h"
#include "views/cluttermediaplayer.h"

#include "assert.h"

Control::Control():
_view(NULL),
_rootMenu(NULL),
_modules(NULL)
{
  

}

Control::~Control(){
  
}

Control * Control::getInstance (  ){
  static Control * _instance = NULL;
  if(_instance == NULL)
    _instance = new Control();
  
  return _instance;
}

Menu * Control::getRootMenu (  ){
  return _rootMenu;
}

View * Control::getView (  ){
  return _view;
}

bool Control::keypressed ( int key ){
  log("Control::keypressed(%i)", key);
  switch (key){
    case 65307: /* Escape */
      // If we are at root don't dispatch the Menu action
      if(getSelectedMenu() == getRootMenu())
        getView()->close();
      else
        getSelectedMenu()->doAction(Menu::MENU);
      break;
    case 65361: /* Left */
      getSelectedMenu()->doAction(Menu::LEFT);
      break;
    case 65362: /* Up */
      getSelectedMenu()->doAction(Menu::UP);
      break;
    case 65363: /* Right */
      getSelectedMenu()->doAction(Menu::RIGHT);
      break;
    case 65364: /* Down */
      getSelectedMenu()->doAction(Menu::DOWN);
      break;
    case 65293: /* Enter */
      getSelectedMenu()->doAction(Menu::OK);
      break;
    
    default:
      int tmp = (int)*SETTINGS->getString("/apps/sofa/keybindings/menu");
      if (key == tmp){
        // If we are at root don't dispatch the Menu action
        if(getSelectedMenu() == getRootMenu())
          getView()->close();
        else
          getSelectedMenu()->doAction(Menu::MENU);
        return true;
      }
      
      tmp = (int)*SETTINGS->getString("/apps/sofa/keybindings/ok");
      if (key == tmp){
        getSelectedMenu()->doAction(Menu::OK);
        return true;
      }
      
      tmp = (int)*SETTINGS->getString("/apps/sofa/keybindings/left");
      if (key == tmp){
        getSelectedMenu()->doAction(Menu::LEFT);
        return true;
      }
      
      tmp = (int)*SETTINGS->getString("/apps/sofa/keybindings/up");
      if (key == tmp){
        getSelectedMenu()->doAction(Menu::UP);
        return true;
      }
      
      tmp = (int)*SETTINGS->getString("/apps/sofa/keybindings/right");
      if (key == tmp){
        getSelectedMenu()->doAction(Menu::RIGHT);
        return true;
      }
      
      tmp = (int)*SETTINGS->getString("/apps/sofa/keybindings/down");
      if (key == tmp){
        getSelectedMenu()->doAction(Menu::DOWN);
        return true;
      }
        
      return false;
  }
  return true;
}

void Control::setRootMenu ( MainMenu * root ){
  _rootMenu = static_cast<SelectionMenu*>(root);
}

void Control::setView ( View * view ){
  _view = view;
}

Menu * Control::getSelectedMenu (  ){
  if(_rootMenu->isSelected()){
    return _rootMenu;
  }else{
    return  getSelectedMenu(_rootMenu);;
  }
    
}

Menu * Control::getSelectedMenu (Menu * menu){
  if(!menu)
    return NULL;
  
  if(menu->isSelected()){
    return menu;
  }
  SelectionMenu * list = dynamic_cast<SelectionMenu*>(menu);
  if(list){   
    for(int j = 0; j <= list->size(); j++){
      Menu * ret = getSelectedMenu(list->get(j));
      if(ret)
        return ret;
    }
  }
  return NULL;
    
}

void Control::init (  int argc, char * argv[]  ){
  log("Initializing Control");
  /** @todo Create Settings */
  _modules = new ModuleManager();
  
  MainMenu * main = new MainMenu();
  main->setTitle("Sofa");
  main->setSubtitle("Main-menu");
  setRootMenu(main);
  
  // Init the view
  ClutterView *view = new ClutterView();
  view->init(argc, argv);
  _view = view;
  view->signal_keypressed.connect( sigc::mem_fun( this, &Control::keypressed) );
  
  
  
  _modules->loadModules( true );
  
  //Add sub menus from the modules to the Main Menu
  
  for(int i = 0; i < _modules->count(); i++){
    Module * mod = _modules->getModule(i);
    if (mod){
       _rootMenu->add(mod->getMenu());
    }
  }
  
  //Detect if no modules were loaded 
  if(_rootMenu->size() == 0 ){
    Menu * menu = new ListMenu();
    menu->setTitle(_("No modules"));
    menu->setSubtitle(_("Use the configuration tool to activate them."));
    menu->setImagePath("/usr/share/pixmaps/gnibbles/bonus1.svg");
    _rootMenu->add(menu);
  }
  main->signal_changed.connect( sigc::mem_fun( getView(), &View::update) );
  
  main->select();
  log("Initializing Control - Done");
}

void Control::uninit (  ){
  _modules->unloadModules( );
}

MediaPlayer * Control::getMediaPlayer (  ){
  // Init the media player
  ClutterMediaPlayer *player = new ClutterMediaPlayer();
  player->init(0, NULL);
  _mediaPlayer = player;
  return _mediaPlayer;
}
