/* 
 * Copyright (C) 2007 Pierre-Luc Beaudoin
 * 
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation; either 
 * version 2 of the License, or (at your option) any later 
 * version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
#include "menu.h"

#include <assert.h>
#include <math.h>

Menu::Menu():
_selected(false),
_percentLoaded(0),
_parent(NULL){
  _title    = NULL;
  _subtitle = NULL;
  _imagePath = NULL;
}

Menu::~Menu(){
  g_free (_title);
  g_free (_subtitle);
  g_free (_imagePath);
}

void Menu::setTitle ( const gchar * title ){
  _title = g_strdup(title);
  signal_changed(Menu::PROPERTIES_CHANGED);
}

const gchar * Menu::getTitle (  ){
  return _title;
}

void Menu::setSubtitle (const gchar * subtitle ){
  _subtitle = g_strdup(subtitle);
  signal_changed(Menu::PROPERTIES_CHANGED);
}

const gchar * Menu::getSubtitle (  ){
  return _subtitle;
}

float Menu::getPercentLoaded (  ){
  return _percentLoaded;
}

void Menu::setPercentLoaded ( float percent ){
  if(fabs(percent - 1.0) < 0.001 ){  // End of loading time, change current selected menu
    Menu * parent = getParent();
    if(parent)
      parent->unselect();
    select();
    _percentLoaded = 0;
    
    //signal_changed.clear();  // This connection is no more needed
    
  } else{
    _percentLoaded = percent;
    signal_changed(Menu::LOADING_CHANGED);
  }
  
}

void Menu::select (  ){
  _selected = true;
  signal_changed(Menu::SELECTED_MENU_CHANGED);
}

void Menu::unselect (  ){
  _selected = false;
  // No update signal here, only the select() is broadcast
}

bool Menu::isSelected (  ){
  return _selected;
}

void Menu::setParent ( Menu * parent){
  _parent = parent;
  signal_changed(Menu::PROPERTIES_CHANGED);
}

Menu * Menu::getParent (){
  return _parent;
}

const gchar * Menu::getImagePath(){
  return _imagePath;
}

void Menu::setImagePath(const gchar * path){
  _imagePath = g_strdup(path);
  signal_changed(Menu::PROPERTIES_CHANGED);
}
