/* 
 * Copyright (C) 2007 Pierre-Luc Beaudoin
 * 
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation; either 
 * version 2 of the License, or (at your option) any later 
 * version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
#ifndef SELECTIONMENU_H
#define SELECTIONMENU_H

#include "../menu.h"
#include <list>
/**
 * @class SelectionMenu
 * @brief A menu where multiple choices are available
 */

class SelectionMenu: public Menu {
  // Attributes
  private:
    
    std::list<Menu *> _list;
    int _highlight;
  // Operations
  public:
    SelectionMenu();
     
    /**
     * @brief Adds a sub-menu to the list.
     * @param menu The Menu to be added to the list.
     */
    void add ( Menu * menu );
    
    /**
     * @brief Returns the sub-menu at given position in the list.
     * @param pos The position in the list.
     * @return The sub-menu at given position in the list.
     */
    Menu * get ( int pos );
    
    /**
     * @brief Clears the list of sub-menus.
     */
    void clear (  );
    
    /**
     * @brief Returns the number of sub-menus.
     * @return The number of sub-menus.
     */
    int size (  );
    
    void highlight ( int pos );
    Menu * getHighlighted (  );
    int getHighlightedPos (  );
   
   
};

#endif
