/* 
 * Copyright (C) 2007 Pierre-Luc Beaudoin
 * 
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation; either 
 * version 2 of the License, or (at your option) any later 
 * version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
#ifndef MEDIAMENU_H
#define MEDIAMENU_H

#include "../menu.h"

#include <glib.h>

class PlayerState;
class MediaPlayer;
class Playlist;
class Clip;

/**
 * @class MediaMenu
 * @brief A menu to play multimedia content
 *
 * This class is the base class to all media playing menus.  Its behavior is 
 * based on the State design pattern.  It knows the following three states:
 * <ul>
 *  <li><b>Playing</b>: The menu is currently playing some content. It is 
 implemented in PlayingState.
 </li>
 *  <li><b>Paused</b>: The playback is currently paused. It is 
 implemented in PausedState.</li>
 *  <li><b>Loading</b>: The content is currently being loaded or buffered. It is 
 implemented in LoadingState.</li>
 * </ul>  
 * At initialisation, the menu is automatically placed in Playing state.  Passing
 * from one state to the next is done using PlayerState::doEvent().  An event 
  (PlayerState::Events) is a converted action (Menu::Actions): based on the action received from the 
  user, each state can convert the proper action for their state (see 
  PlayerState::getEventFromAction()).  The event is then executed with 
  PlayerState::doEvent().  
 */
 
 
class MediaMenu: public Menu {
  // Associations
  // Attributes
  private: 
    void tick();
    void currentChanged();
    guint _controls_to;
    bool _controls_on;
    
  protected:
    PlayerState * _state;
    MediaPlayer * _mediaPlayer;
    Playlist * _playlist;
    
  // Operations
  public:
    MediaMenu();
    void doAction ( Actions anAction );
    void setMediaPlayer ( MediaPlayer * mp );
    MediaPlayer * getMediaPlayer ( );
    void play ( Playlist * pl );
    Clip * getCurrentClip();
    void controlsOff();
    void controlsOn();
    
};

#endif
