/* 
 * Copyright (C) 2007 Pierre-Luc Beaudoin
 * 
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation; either 
 * version 2 of the License, or (at your option) any later 
 * version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef PLAYERSTATE_H
#define PLAYERSTATE_H

#include "musicmenu.h"
#include "../../views/mediaplayer.h"

class PlayingState;
class PausedState;
class LoadingState;
class SeekForwardState;
class SeekBackwardState;

class PlayerState : public sigc::trackable  {
  // Associations
  // Attributes
  public:
    /** @brief Emited when the current playlist is finished.
     */ 
    sigc::signal<void>  signal_end;
     
    /** @brief Emited when the state is changed.
     * 
     * This signal should be emited when any change happens. 
     */ 
    sigc::signal<void>  signal_changed;
     
    enum Events {
      PLAY = 0,
      NEXT,
      PREVIOUS,
      SEEK_BACKWARD,
      SEEK_FORWARD,
      PAUSE,
      VOLUME_UP,
      VOLUME_DOWN,
      LOADING,
      LOADING_DONE
    };
     
  private:
    
    
  protected:
    static PlayerState  * _currentState;
  
    static PlayingState * _playingState;
    static PausedState  * _pausedState;
    static LoadingState * _loadingState;
    
    static MediaPlayer  * _mediaPlayer;
    static Playlist     * _playlist;
    
  // Operations
  public:
    PlayerState();
    static PlayerState * getInitialState();
    PlayerState * getCurrentState();
    void init (MediaPlayer * mp, Playlist * playlist );
    bool isPlaying (  );
    bool isPaused (  );
    bool isLoading (  );
    virtual void doEvent ( Events event );
    
    virtual Events getEventFromAction ( Menu::Actions Action ) = 0;
    virtual PlayerState * getNextState ( Events event ) = 0;
    virtual void enter (  ) = 0;
    virtual void force (  ) = 0;
};

#endif
