/* 
 * Copyright (C) 2007 Pierre-Luc Beaudoin
 * 
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation; either 
 * version 2 of the License, or (at your option) any later 
 * version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
 
#include "ivpmodule.h"
#include "ivpconfigwindow.h"


#include "../../sofa-i18n.h"
#include "../../fileutils.h"
#include "../../menus/viewers/videomenu.h"
#include "../../views/intplaylist.h"
#include "../../views/cluttermediaplayer.h"
#include "../../clip.h"
#include "../../md5.h"

#include <assert.h>
#include <sigc++/bind.h>


IVPModule::IVPModule(){
  _root = NULL;
  _homedir =  g_get_home_dir();
}

const gchar * IVPModule::getName (  ){
  return _("Internal Video Player");;
}

const gchar * IVPModule::getSettingName (  ){
  return "intvideoplayer";
}

const gchar * IVPModule::getVersion (  ){
  return "0.0.17";
}

const gchar * IVPModule::getAuthor (  ){
  return "Pierre-Luc Beaudoin";
}

const gchar * IVPModule::getDescription (  ){
  return _("This module demonstrates the modular architecture of Sofa");
}

void IVPModule::unload (  ){
assert(!"Unimplemented");
}

Menu* IVPModule::getMenu (  ){
  if(!_root){
     _root = new ListMenu( );
    _root->setTitle(_("Video Player..."));
    char * sub = new char[100];
    sprintf(sub, _("Video files from %s"), _homedir);
    _root->setSubtitle(sub);
    _root->setImagePath("/usr/share/icons/Tango/scalable/mimetypes/video.svg");
    _root->signal_load.connect(sigc::mem_fun(this, &IVPModule::loadSubMenu));
    delete sub;
  }
  return _root;
}
    
/*void * thread_function( void *ptr )
{ 
  IVPModule * mod = (IVPModule*) ptr;
  printf("Thread \n");
  mod->visitHome();
  printf("Done \n");
  
}*/

void IVPModule::loadSubMenu(){
  SelectionMenu * list = reinterpret_cast<SelectionMenu *>(_root);
  VideoMenu * vid = NULL;
  
  if(list->size()== 0){
    GThread          *Thread1;
    GError           *err1 = NULL ;
    char *message1 = "Thread 1";
    list->setPercentLoaded(0.1);
    visit(&list, _homedir, _homedir);
    /*Thread1 = g_thread_create((GThreadFunc)thread_function, (void *)this, TRUE, &err1);
    g_thread_yield();
  	g_thread_join(Thread1);*/
  }
  
 _root->setPercentLoaded(1.0);
  
}

/*void IVPModule::visitHome(){
  SelectionMenu * list = reinterpret_cast<SelectionMenu *>(_root);
  VideoMenu * vid = NULL;
  
  if(list->size()== 0){
    visit(&list, _homedir.c_str(), _homedir.c_str());
  }
  //list->setPercentLoaded(1.0);
}*/


void IVPModule::visit(SelectionMenu ** list, const gchar * path, const gchar * name){
  VideoMenu * vid = NULL;
  GDir *dir;
  const gchar *file;
  const char  *ext;
  gchar * completepath;

  dir = g_dir_open(path, 0, NULL);

  if (dir){
  	while ((file = g_dir_read_name(dir))){
	    if(!isHidden(file)){
      		completepath = g_build_filename(path, file, NULL);
      		
        	
	
      		if(g_file_test(completepath, G_FILE_TEST_IS_DIR)){
      		  //Recursively run through sub dirs
      		  SelectionMenu * tmp = NULL;
      		  visit(&tmp, completepath, file);
      		  
      		  //If the returned Menu is not null, add it
      		  if(tmp != NULL){
      		    if(*list == NULL){
      		      (*list) = new ListMenu();
                (*list)->setTitle(name);
                (*list)->setSubtitle(_("Directory"));
                (*list)->setImagePath("/usr/share/icons/Tango/scalable/places/folder.svg");
                (*list)->signal_load.connect(sigc::bind(sigc::mem_fun(*this, &IVPModule::showDir),*list));
              }
      		    (*list)->add(tmp);
      		  }
      		} else {
      		  //printf("Fichier: %s\n", file);
        		if(isAVideoFile(file)){
        		  completepath = g_build_filename(path, file, NULL);
        		  if(*list == NULL){
        		    (*list) = new ListMenu();
                (*list)->setTitle(name);
                (*list)->setSubtitle(_("Directory"));
                (*list)->setImagePath("/usr/share/icons/Tango/scalable/places/folder.svg");
                (*list)->signal_load.connect(
                  sigc::bind(sigc::mem_fun(*this, &IVPModule::showDir),*list)
                  );
              }
              char * uri = g_filename_to_uri(completepath, NULL, NULL);
              
          		//Compute MD5 to find thumbnail
          		md5_state_t * state= g_new0(md5_state_t, 1);
            	md5_byte_t * digest = g_new0(md5_byte_t,16);
            	int  * di = new int;
              char * thum = new char [32];
              char * ptr;
              
            	md5_init(state);
            	md5_append(
            	  state, 
            	  (const md5_byte_t *)uri, 
            	  strlen(uri));
            	md5_finish(state, digest);
            	
            	ptr = thum;
              for (int x = 0; x < 16; x++)
                ptr += sprintf(ptr, "%2.2x", digest[x]);
              g_free(uri);
              delete di;
                                    
            	g_free(digest);
            	g_free(state);
            	char * thumb = new char [250];
            	sprintf(thumb, "%s/.thumbnails/normal/%s.png", g_get_home_dir(), thum);
            	//printf("%s\n", thumb);
              vid = new VideoMenu();
              vid->setTitle(stripExtension(file).c_str());
              vid->setSubtitle(completepath);
              vid->setImagePath(thumb);//"/usr/share/icons/Tango/scalable/mimetypes/video.svg"
              vid->signal_load.connect(sigc::bind(sigc::mem_fun(*this,
                &IVPModule::showVideoMenu),vid, completepath));
              (*list)->add(vid);
              
            }
          }
          //g_free(completepath); // Cannot free because the string is used by sigc::bind
        }

  	}

  	g_dir_close(dir);
  }
  
}

void IVPModule::showDir(SelectionMenu*menu){
  menu->setPercentLoaded(1.0);
}

void IVPModule::showVideoMenu(VideoMenu*menu, const gchar * filepath){
  //This is a short fix to unresolved symbols: we can't access Control from whitin 
  //modules, therefore, I can get the default MediaPlayer.
  ClutterMediaPlayer * cmpl = new ClutterMediaPlayer();
  cmpl->init(0, NULL);
  menu->setMediaPlayer(cmpl);

  menu->setPercentLoaded(0.1);
  IntPlaylist * pl = new IntPlaylist();
  pl->init(0, NULL, cmpl);
  Clip * c = new Clip();
  std::string uri = "file://";
  uri.append(filepath);
  c->setURI(uri);
  pl->append(c);

  menu->setPercentLoaded(1.0);
  menu->play(pl);
}

ConfigWindow * IVPModule::getConfigWindow (  ) {
  IVPConfigWindow * ret = new IVPConfigWindow();
  return ret;
}
