/* 
 * Copyright (C) 2007 Pierre-Luc Beaudoin
 * 
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation; either 
 * version 2 of the License, or (at your option) any later 
 * version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
 
#include "rbplaylist.h"
#include "rbmediaplayer.h"
#include "../../clip.h"

#include <assert.h>

static void  uriChanged_cb (DBusGProxy *proxy, 
                      const gchar* tick,
                      Playlist * pl  )
{
  printf("Song changed");
  pl->signal_currentChanged.emit();
}


void RBPlaylist::init ( int argc, char* argv[], MediaPlayer * mediaPlayer  ){
  _mediaPlayer = mediaPlayer;
  
  GError *error = NULL;
  DBusGConnection *bus;
  DBusGProxy *bus_proxy;
  guint start_service_reply;
  
  /* get dbus connection */
	bus = dbus_g_bus_get (DBUS_BUS_SESSION, &error);
	if (error)
		assert(!"Couldn't create the dbus connection");
  
  /* if rhythmbox is not started, start it */
  bus_proxy = dbus_g_proxy_new_for_name (bus,
						       "org.freedesktop.DBus",
						       "/org/freedesktop/DBus",
						       "org.freedesktop.DBus");

	if (!dbus_g_proxy_call (bus_proxy, "StartServiceByName", &error,
					G_TYPE_STRING, "org.gnome.Rhythmbox",
					G_TYPE_UINT, 0,
					G_TYPE_INVALID,
					G_TYPE_UINT, &start_service_reply,
					G_TYPE_INVALID)) {
			g_warning ("%s", error->message);
			exit (1);
		}
  
  _shell = dbus_g_proxy_new_for_name_owner (bus,
					      "org.gnome.Rhythmbox",
					      "/org/gnome/Rhythmbox/Shell",
					      "org.gnome.Rhythmbox.Shell",
					      &error);
	if (error) {
		_shell = NULL;
		_player = NULL;
		_plman = NULL;
		assert(!"Couldn't create the shell proxy");
	}
  
	_player = dbus_g_proxy_new_from_proxy (_shell,
						     "org.gnome.Rhythmbox.Player",
						     "/org/gnome/Rhythmbox/Player");
	if (_player == NULL) {
		g_object_unref (G_OBJECT (_shell));
		_shell = NULL;
		_player = NULL;
		_plman = NULL;
		assert(!"Couldn't create the player proxy");
	}
	
	_plman = dbus_g_proxy_new_from_proxy (_shell,
						     "org.gnome.Rhythmbox.PlaylistManager",
						     "/org/gnome/Rhythmbox/PlaylistManager");
	if (_plman == NULL) {
		g_object_unref (G_OBJECT (_shell));
		_shell = NULL;
		_player = NULL;
		assert(!"Couldn't create the playlist manager proxy");
	}
	g_clear_error (&error);
	
	dbus_g_proxy_add_signal (_player, "playingUriChanged", G_TYPE_STRING, G_TYPE_INVALID);
  dbus_g_proxy_connect_signal (_player, "playingUriChanged", G_CALLBACK(uriChanged_cb), (void*)this, NULL);
	
}

void RBPlaylist::append ( Clip* clip ){
}

void RBPlaylist::next (  ){
  printf("RBPlaylist::next\n");
  GError *error = NULL;
  dbus_g_proxy_call (_player, "next", &error, G_TYPE_INVALID, G_TYPE_INVALID);
  if (error) {
		assert(!"Couldn't set the player to next");
  }
}

void RBPlaylist::previous (  ){
  printf("RBPlaylist::previous\n");
  GError *error = NULL;
  dbus_g_proxy_call (_player, "previous", &error, G_TYPE_INVALID, G_TYPE_INVALID);
  if (error) {
		assert(!"Couldn't set the player to next");
  }
}

void RBPlaylist::clear (  ){
}

Clip * RBPlaylist::getCurrentClip (  ){
  RBMediaPlayer  * pl = NULL;;
  if(pl = dynamic_cast<RBMediaPlayer*>(_mediaPlayer)){
    return pl->getClip();
  }
  return new Clip();
}

int RBPlaylist::getCurrentPos (  ){
  return 0;
}

int RBPlaylist::size (  ){
  return -1;
}
