/* 
 * Copyright (C) 2007 Pierre-Luc Beaudoin
 * 
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation; either 
 * version 2 of the License, or (at your option) any later 
 * version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "cluttermediaplayer.h"
#include <clutter-gst/clutter-gst.h>
#include <gst/gst.h>

#include <assert.h>

//Position callback
void 
tick (GObject      *object,
      GParamSpec   *pspec,
      ClutterMediaPlayer * pl)
{
  if(pl->getProgress() == 1){
    pl->signal_paused.emit();
  }
  pl->signal_tick.emit();
}

//Position callback
void 
playpause (GObject      *object,
      GParamSpec   *pspec,
      ClutterMediaPlayer * pl)
{
  if(pl->isPlaying())
    pl->signal_playing.emit();
  else
    pl->signal_paused.emit();

}

void ClutterMediaPlayer::init ( int argc, char* argv[] ){
  clutter_init (&argc, &argv);
  gst_init (&argc, &argv);
  _clutterMedia = clutter_gst_video_texture_new ();
  g_signal_connect (_clutterMedia,
		    "notify::position",
		    G_CALLBACK (tick),
		    (void*) this);
}

void ClutterMediaPlayer::play (  ){
  clutter_media_set_playing (CLUTTER_MEDIA(_clutterMedia), TRUE);
}

void ClutterMediaPlayer::pause (  ){
  clutter_media_set_playing (CLUTTER_MEDIA(_clutterMedia), FALSE);
}

void ClutterMediaPlayer::setURI ( std::string uri ){
  printf("URI: %s", uri.c_str());
  
  clutter_media_set_uri(CLUTTER_MEDIA(_clutterMedia), uri.c_str());
}

float ClutterMediaPlayer::getProgress (  ){
  int duration = clutter_media_get_duration( CLUTTER_MEDIA(_clutterMedia) );
  if(duration == 0)
    return 0;
  else 
    return (clutter_media_get_position(CLUTTER_MEDIA(_clutterMedia)) / 
            ((float) duration));
}

void ClutterMediaPlayer::setProgress ( float progress ){
  clutter_media_set_position(
    CLUTTER_MEDIA(_clutterMedia),
    (int) (clutter_media_get_duration( CLUTTER_MEDIA(_clutterMedia) ) * progress)
  );
}

bool ClutterMediaPlayer::isPlaying (  ){
  return clutter_media_get_playing( CLUTTER_MEDIA(_clutterMedia) );
}

int ClutterMediaPlayer::getDuration (  ){
  return clutter_media_get_duration( CLUTTER_MEDIA(_clutterMedia) );
}

float ClutterMediaPlayer::getVolume(  ){
  return clutter_media_get_volume(CLUTTER_MEDIA(_clutterMedia));
}

void ClutterMediaPlayer::setVolume ( float percent ){
  clutter_media_set_volume(
    CLUTTER_MEDIA(_clutterMedia),
    percent
  );
}

ClutterActor * ClutterMediaPlayer::getClutterMediaTexture (  ){
  return _clutterMedia;
}
