/* 
 * Copyright (C) 2007 Pierre-Luc Beaudoin
 * 
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation; either 
 * version 2 of the License, or (at your option) any later 
 * version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
#ifndef CLUTTERVIEW_H
#define CLUTTERVIEW_H

#include "view.h"

#include <clutter/clutter.h>
#include <string>
#include "../menus/selecters/listmenu.h"

// Forward declaration

class MusicMenu;
class VideoMenu;

/**
 * @class ClutterView
 * @brief The Clutter based view.
 *
 * This view uses <a href="http://www.clutter-project.org">Clutter-0.2</a> to 
 * render the screen. This library is mainly a wrapper around OpenGL with a GTK+ 
 * code style.  
 */
typedef struct CMenu
{
  ClutterActor * background;
  ClutterActor * title;
  ClutterActor * subtitle;
  ClutterTexture * image;
  
  ClutterActor * menu_options;
  
  ClutterTimeline *fade_in, 
                  * fade_out, 
                  * slide, 
                  * loading;
  
  ClutterTimeline * controls_tl;
  ClutterActor * controls;
  
}
CMenu;
    
    
class ClutterView: public View {
  // Associations
  // Attributes
  private: 
    CMenu           * _current;
    Menu            * _currentMenu;
    int               _currentPos; 
    ClutterActor    * _stage;
    
    std::string       bigMenuFont, 
                      smallMenuFont, 
                      logoFont;
    ClutterColor      menuColor, 
                      selectedColor; 
    int               bigFontLineHeight,
                      smallFontLineHeight;
    unsigned int      screenWidth,
                      screenHeight;
    // Menu builders                  
    CMenu  *         buildLogo();
    void             createBackground (ClutterActor * bg);
    ClutterActor  *  buildSelectionMenu (SelectionMenu* menu);
    ClutterActor  *  buildViewerMenu(Menu* menu);
    ClutterActor  *  buildMusicMenu(MusicMenu * menu);
    ClutterActor  *  buildVideoMenu(VideoMenu * menu);
    
    // Menu updaters
    void            updateSelectionMenu(SelectionMenu * selmenu);
    void            updateLoadingSelectionMenu(SelectionMenu * selmenu);
    void            updateMusicMenu(MusicMenu * musicmenu);
    void            updateVideoMenu(VideoMenu * musicmenu);
    // Action doers
    void            goTo(Menu * m);
    void            showControls();
    void            hideControls();
  // Operations
  public:
    ClutterView();
    void init( int argc, char * argv[] );
    void update ( Menu::Updates up );
    void show (  );
    void close (  );
    int  x(float percent);
    int  y(float percent);
    int  getScrollLineHeight();
    ClutterActor * getStage();
    
    static const int LINE_SPACING     = 20; // Spacing between lines in menus
    static const int SEL_CHANGE_DUR   = 10; // Number of frames of selection change animation
    static const int MENU_CHANGE_DUR  = 30; // Number of frames of selection change animation
    static const int LOGO_DUR         = 30; // Number of frames of selection change animation
    
    
};

#endif
