/* 
 * Copyright (C) 2007 Pierre-Luc Beaudoin
 * 
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation; either 
 * version 2 of the License, or (at your option) any later 
 * version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
#ifndef VIEW_H
#define VIEW_H

#include "../menus/menu.h"

#include <sigc++/trackable.h>
#include <sigc++/signal.h>

/**
 * @class View
 * @brief The base class for all views.
 *
 * A view is a graphical representation of the menus.  More precisely, a view is 
 * a window.  This base class sets a common interface to allow the creation of
 * many different views base on different toolkits.
 */

class View: public sigc::trackable {
  // Associations
  // Attributes
  public:
    sigc::signal<bool,int> signal_keypressed;
    
  // Operations
  public:
    /**
     * @brief Initialise the underlying toolkit.
     * @param argc The argument count passed to main().
     * @param argv The argument vector passed to main().
     */
    virtual void init( int argc, char * argv[] ) = 0;
    
    /**
     * @brief Updates the content of the window.
     * 
     * This method will be called each time that the menus changes.  Basicaly, 
     * it is connected to Menu::signal_changed. 
     */
    virtual void update ( Menu::Updates up ) = 0;
    
    /**
     * @brief Opens the window.
     * 
     * This call should be blocking.  When the window is closed, this method will
     * return.
     */
    virtual void show (  ) = 0;
    
    
    /**
     * @brief Closes the window.
     */
    virtual void close (  ) = 0;
};

#endif
